# -*- coding: utf-8 -*-

__uri__ = "https://simmer.readthedocs.io"
__author__ = "Arjun B. Savel, Lea Hirsch, Courtney D. Dressing, David R. Ciardi"
__email__ = "asavel@gmail.com"
__license__ = "MIT"
__version__ = "0.3.5"
__release__ = "0.3.5-beta"
__description__ = "Stellar Image Maturation via Efficient Reduction"
